using System;
using System.Collections.Generic;
using System.IO;
using UnityEngine;
using UnityEngine.Events;

namespace Mod
{	
    public class Mod
    {	
		public static void Main()
		{			
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Accumulator"),
					NameOverride = "Fusion Generator",
					DescriptionOverride = "20 times more powerful than an industrial generator. Gets extremely hot; requires water cooling for prolonged continuous use or it will break down!", 
					CategoryOverride = ModAPI.FindCategory("Machinery"),
					ThumbnailOverride = ModAPI.LoadSprite("fusionthumb.png"),
					AfterSpawn = (Instance) => 
					{
						var spr = Instance.GetComponent<SpriteRenderer>();
						spr.sprite = ModAPI.LoadSprite("fusionsprite.png");
						Instance.FixColliders();
						
						var gen = Instance.GetComponent<GeneratorBehaviour>();
						gen.TargetCharge = 20000f;
						gen.TargetEnergyReachSpeed = 1f;
						gen.Vibrate = true;
						gen.VibrateForce = 100f;
						gen.VibrateRotationSpeed = -40f;
						var dmg = Instance.GetComponent<DamagableMachineryBehaviour>();
						dmg.TemperatureThreshold = 50000f;
						dmg.FireDamageThreshold = 50000f;
						var phys = Instance.GetComponent<PhysicalBehaviour>();
						phys.TrueInitialMass = 15;
						phys.InitialMass = 15;
					}
				}
			);
			
			
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Accumulator"),
					NameOverride = "Baryonic Conversion Generator",
					DescriptionOverride = "Antimatter reactor, 100 times more powerful than an industrial generator. Has a higher heat tolerance than the fusion generator but still don't touch!", 
					CategoryOverride = ModAPI.FindCategory("Machinery"),
					ThumbnailOverride = ModAPI.LoadSprite("baryonthumb.png"),
					AfterSpawn = (Instance) => 
					{
						var spr = Instance.GetComponent<SpriteRenderer>();
						spr.sprite = ModAPI.LoadSprite("baryonsprite.png");
						Instance.FixColliders();
						
						var gen = Instance.GetComponent<GeneratorBehaviour>();
						gen.TargetCharge = 100000f;
						gen.TargetEnergyReachSpeed = 5f;
						gen.Vibrate = true;
						gen.VibrateForce = 200f;
						gen.VibrateRotationSpeed = -50f;
						var dmg = Instance.GetComponent<DamagableMachineryBehaviour>();
						dmg.TemperatureThreshold = 1000000f;
						dmg.FireDamageThreshold = 1000000f;
						dmg.OnDamaged.AddListener(() => {
							ExplosionCreator.Explode(new ExplosionCreator.ExplosionParameters
							{
								Position = Instance.GetComponent<Rigidbody2D>().transform.position,
								CreateParticlesAndSound = true,
								LargeExplosionParticles = true,
								DismemberChance = 0.6f,
								FragmentForce = 20,
								FragmentationRayCount = 128,
								Range = 30
							});
						});
						var phys = Instance.GetComponent<PhysicalBehaviour>();
						phys.TrueInitialMass = 40;
						phys.InitialMass = 40;
					}
				}
			);
			
			ModAPI.Register(
				new Modification()
				{
					OriginalItem = ModAPI.FindSpawnable("Accumulator"),
					NameOverride = "Singularity Energy Extractor",
					DescriptionOverride = "Black hole generator. A thousand times more powerful than an industrial generator, also a thousand times more dangerous. Use with caution.", 
					CategoryOverride = ModAPI.FindCategory("Machinery"),
					ThumbnailOverride = ModAPI.LoadSprite("singularitythumb.png"),
					AfterSpawn = (Instance) => 
					{
						var spr = Instance.GetComponent<SpriteRenderer>();
						spr.sprite = ModAPI.LoadSprite("singularitysprite.png");
						Instance.FixColliders();
						
						var gen = Instance.GetComponent<GeneratorBehaviour>();
						gen.TargetCharge = 1000000f;
						gen.TargetEnergyReachSpeed = 7f;
						gen.Vibrate = false;
						//gen.VibrateForce = 200f;
						//gen.VibrateRotationSpeed = -50f;
						var dmg = Instance.GetComponent<DamagableMachineryBehaviour>();
						dmg.TemperatureThreshold = 10000000f;
						dmg.FireDamageThreshold = 10000000f;
						dmg.OnDamaged.AddListener(() => {
							GameObject.Instantiate(ModAPI.FindSpawnable("Decimator").Prefab.GetComponent<DecimatorBehaviour>().BlackHole, Instance.transform.position, Quaternion.identity);
							UnityEngine.Object.Destroy(Instance);
						});
						var phys = Instance.GetComponent<PhysicalBehaviour>();
						phys.TrueInitialMass = 80;
						phys.InitialMass = 80;
					}
				}
			);
		}
    }
}